package xyz.tbvns.Models;

import com.badlogic.gdx.graphics.g3d.Model;
import com.badlogic.gdx.graphics.g3d.ModelInstance;
import lombok.Data;
import lombok.NoArgsConstructor;
import xyz.tbvns.LevelInfo;

@Data
@NoArgsConstructor
public class GDInstance {
    public GDInstance(ModelInstance instance, String name, String id, GDModel gdModel) {
        this.instance = instance;
        this.name = name;
        this.id = id;
        this.animation = false;
        this.gdModel = gdModel;
    }

    public GDInstance(ModelInstance instance, int fps, int frameSkip, double start, String name, String id, Animation animation) {
        this.instance = instance;
        this.fps = fps;
        this.frameSkip = frameSkip;
        this.start = start;
        this.name = name;
        this.id = id;
        this.animation = true;
        this.animationObject = animation;
        this.followCamera = false;
    }

    public void updateFrame(double percent) {
        instance = new ModelInstance(animationObject.getModelFrames().get(getFrameOnPercent(percent)), instance.transform);
        instance.userData = this;
    }

    //TODO: Make this not break if the speed change in the middle of the animation
    public int getFrameOnPercent(double percent) {
        double usablePercent = percent - (start / 100);
        if (usablePercent > 0) {
            float max = LevelInfo.getLevel().getMaxPos();
            double pos = max * usablePercent;
            double speed = LevelInfo.getLevel().getSpeedAt((float) percent);

            double distancePerFrame = (speed / fps) * 10;
            int frame = (int) (pos / distancePerFrame);

            if (frame < animationObject.getModelFrames().size()) {
                return frame;
            }
            return animationObject.getModelFrames().size() - 1;
        }
        return 0;
    }

    private ModelInstance instance;
    private boolean animation;
    private double start;
    private int current;
    private int fps, frameSkip;
    private String name, id;
    private boolean followCamera;
    private Animation animationObject;
    private GDModel gdModel;
}
